/****************************************************
 *  Bachelor Project:                               *
 *  External Memory Minimum Spanning Trees          *
 *  by Dominik Schultes <mail@dominik-schultes.de>  *
 *                                                  *
 *                mst.h                             *
 *                                                  *
 *  April 13, 2003                                  *
 ****************************************************/


#ifndef MST_H
#define MST_H

#include <iostream>

#include "edgeVector.h"
#include "relabeledEdge.h"

/**
   Represents a Minimum Spanning Tree (MST) of a graph.
*/
class MST
{
    /** Writes a string representation of the MST to an output stream. */
    friend std::ostream& operator<<( std::ostream& os, MST &mst ) {
	os << mst.noOfEdges() << std::endl;
	for (EdgeCount i=0; i<mst.noOfEdges(); i++)
	    os << mst._mst[i] << std::endl;
	return os;
    }
    
 public:
    /** The default constructor. */
    MST()
	: _mst( 0, 0 ), _totalWeight( 0 )
	{}
    
    /** Returns the number of edges of the MST. */
    EdgeCount noOfEdges() const {return _mst.noOfEdges();}

    /** Returns the sum of the weights of all edges of the MST. */
    EdgeWeightBig totalWeight() const {return _totalWeight;}

    /** Adds an edge to the MST. */
    void add(const RelabeledEdge &edge);
    
    /** Adds an edge to the MST. */
    void add(const RelabeledEdgeWithoutSource &edge);

    /** Adds an edge to the MST. */
    void add(const Edge &edge);
    
 private:
    EdgeVector<Edge,
	       DS_MST_BLOCK_SIZE,
	       DS_MST_NO_OF_PAGES,
	       DS_MST_PAGE_SIZE> _mst;

    EdgeWeightBig _totalWeight;
};


/** Adds an edge to the MST. */
inline void MST::add(const RelabeledEdge &edge)
{
    // The original source and target indices are restored ...
    Edge originalEdge( edge.originalSource(),
		       edge.originalTarget(),
		       edge.weight() );
    
    // ... and the original edge is added to the MST.
    add( originalEdge );
}


/**
   Adds an edge to the MST.
   This is an explicit copy of the add method for a RelabeledEdge
   in order to avoid virtual methods.
*/
inline void MST::add(const RelabeledEdgeWithoutSource &edge)
{
    // The original source and target indices are restored ...
    Edge originalEdge( edge.originalSource(),
		       edge.originalTarget(),
		       edge.weight() );
    
    // ... and the original edge is added to the MST.
    add( originalEdge );
}


/** Adds an edge to the MST. */
inline void MST::add(const Edge &edge)
{
    // add the given edge to the MST
    _mst.push_back( edge );
    
    // update total weight
    _totalWeight += edge.weight();
}


#endif // MST_H
