/****************************************************
 *  Bachelor Project:                               *
 *  External Memory Minimum Spanning Trees          *
 *  by Dominik Schultes <mail@dominik-schultes.de>  *
 *                                                  *
 *                generate.h                        *
 *                                                  *
 *  April 13, 2003                                  *
 ****************************************************/


#ifndef GENERATE_H
#define GENERATE_H

#include "../data_structures/edgeVector.h"

/**
 Generates a complete graph with the given number of nodes.
 The weights of the edges are chosen randomly.
*/
EdgeVector<> * generateCompleteGraph(NodeCount noOfNodes);

/**
   Generates a random graph with the given number of nodes
   and the given number of edges.
   The weights of the edges are chosen randomly, too.
*/
EdgeVector<> * generateRandomGraph(NodeCount noOfNodes, EdgeCount noOfEdges);

/**
   Generates a grid graph with "noOfNodesX * noOfNodesY" nodes.
   The weights of the edges are chosen randomly.
*/
EdgeVector<> * generateGridGraph(NodeCount noOfNodesX, NodeCount noOfNodesY);

/**
   Generates a geometric graph.
   The given number of nodes is placed in a square and each node is connected with
   the given number of nearest neighbours. Duplicates (which come into existence when
   two nodes select each other) are removed.
*/
EdgeVector<> * generateGeometricGraph(NodeCount noOfNodes, NodeCount noOfNeighbours);

/** Returns a random node identifier between 0 and n-1. */
inline NodeID randomNodeID(NodeID n);

/** Returns a random edge weight. */
inline EdgeWeight randomEdgeWeight();

#endif // GENERATE_H
