/****************************************************
 *  Bachelor Project:                               *
 *  External Memory Minimum Spanning Trees          *
 *  by Dominik Schultes <mail@dominik-schultes.de>  *
 *                                                  *
 *                importExport.h                    *
 *                                                  *
 *  April 13, 2003                                  *
 ****************************************************/


#ifndef IMPORTEXPORT_H
#define IMPORTEXPORT_H

#include <iostream>

#include "../data_structures/edgeVector.h"


/**
   Imports an EdgeVector from a file.
   expected format:
   the format which is expected by 'importEdgeVectorListOfEdges' or
   the format which is expected by 'importEdgeVectorAdjList' or
   the format which is expected by 'importEdgeVectorCompressed'
*/
EdgeVector<> * importEdgeVector(const std::string &filename);


/**
   Imports an EdgeVector from an istream.
   expected format:
   
   noOfNodes noOfEdges
   source0 target0 weight0
   source1 target1 weight1
   source2 target2 weight2
   ...

   sourceX and targetX are node indizes; the first used node index should be 0.
*/
EdgeVector<> * importEdgeVectorListOfEdges(std::istream &in);


/**
   Imports an EdgeVector from an istream.
   expected format:
   
   'a'
   noOfNodes
   adjacency list of node 1
   -1
   adjacency list of node 2
   -1
   ...

   Each adjacency list is a list of pairs: node1 weight1 node2 weight2 ...
   The first used node index should be 1.
*/
EdgeVector<> * importEdgeVectorAdjList(std::istream &in);


/**
   Imports an EdgeVector from a file.
   expected format:

   'c'
   noOfNodes noOfEdges
   source0 target0 weight0
   source1 target1 weight1
   source2 target2 weight2
   ...

   sourceX and targetX are node indizes; the first used node index should be 0.
   Each 32-bit number (sourceX, targetX, weightX) should be represented by 4 chars
   and there should be no spaces and newlines after the number of edges.
   If necessary, several files are used so that the file size limit isn't exceeded.
*/
EdgeVector<> * importEdgeVectorCompressed(const std::string &filename);


/**
   Exports an EdgeVector to an ostream.
   created format:

   'a'
   noOfNodes
   adjacency list of node 1
   -1
   adjacency list of node 2
   -1
   ...

   Each adjacency list is a list of pairs: node1 weight1 node2 weight2 ...
   The first used node index is 1.
*/
void exportEdgeVectorAdjList(std::ostream &out, EdgeVector<> &edges);


/**
   Exports an EdgeVector to a file.
   created format:

   'c'
   noOfNodes noOfEdges
   source0 target0 weight0
   source1 target1 weight1
   source2 target2 weight2
   ...

   sourceX and targetX are node indizes; the first used node index is 0.
   Each 32-bit number (sourceX, targetX, weightX) is represented by 4 chars
   and there are no spaces and newlines after the number of edges.
   If necessary, several files are used so that the file size limit isn't exceeded.
*/
void exportEdgeVectorCompressed(const std::string &filename, EdgeVector<> &edges);


/**
   Returns the number of nodes of the graph which is stored in the file
   with the given name.
*/
NodeCount noOfNodesInFile(const std::string &filename);


#endif // IMPORTEXPORT_H
