/* utils.h */

#ifndef UTILS_H
#define UTILS_H

#include "../stxxl/utils/malloc.h"

#include "../buckets/buckets.h"


class Percent
{
 private:
    typedef int percentInt;
    typedef EdgeCount valueInt;
    
 public:
    Percent(valueInt maxValue, bool reverse = false, percentInt step = 2) {
	reinit(maxValue, reverse, step);
    }

    void reinit(valueInt maxValue, bool reverse = false, percentInt step = 2) {
	_maxValue = maxValue;
	_intervalPercent = _maxValue / 100;
	_nextThreshold = _intervalPercent;
	_reverse = reverse;
	_lastPercent = 0;
	_step = step;
	
	if (reverse) _nextThreshold = _maxValue - _intervalPercent;
    }

    void printStatus(valueInt currentValue) {
	if ( ! _reverse) {
	    if (currentValue >= _nextThreshold) {
		_nextThreshold += _intervalPercent;
		printPercent( currentValue / (double)_maxValue * 100 );
	    }
	}
	else {
	    if (currentValue <= _nextThreshold) {
		_nextThreshold -= _intervalPercent;
		printPercent( 100 - (currentValue / (double)_maxValue * 100) );
	    }
	}
    }
	
 private:
    valueInt _maxValue;
    valueInt _intervalPercent;
    valueInt _nextThreshold;
    bool _reverse;
    percentInt _lastPercent;
    percentInt _step;

    void printPercent(double percent) {
	while (percent >= _lastPercent+_step) {
	    _lastPercent+=_step;
	    if (_lastPercent % 10 == 0) {
		std::cout << " " << _lastPercent << "% ";
	    }
	    else {
		std::cout << ".";
	    }
	    std::cout.flush();
	}
    }
};


void memoryUsageForecast(NodeCount noOfNodes,
			 NodeCount noOfNodesInInternalMemory,
			 Buckets<>::BucketID noOfBuckets)
{
    if (noOfNodes <= noOfNodesInInternalMemory) return;
    
    // first external bucket
    int memoryUsage = DS_DEFAULT_BLOCK_SIZE * DS_DEFAULT_NO_OF_PAGES * DS_DEFAULT_PAGE_SIZE;
    // mst
    memoryUsage += DS_MST_BLOCK_SIZE * DS_MST_NO_OF_PAGES * DS_MST_PAGE_SIZE;
    // all external buckets but the first
    memoryUsage += (noOfBuckets-1) * DS_EXT_BLOCK_SIZE * DS_EXT_PAGE_SIZE;
    // internal buckets: first block of each node (16 bytes per edge)
    memoryUsage += DS_EXT_BUCKET_SIZE * DS_INT_EDGES_PER_BLOCK * 16;
    // internal buckets: blocks in the common pool (16 bytes per edge)
    memoryUsage += DS_INT_NO_OF_BLOCKS * DS_INT_EDGES_PER_BLOCK * 16;

    std::cout << "memory usage forecast: " << memoryUsage / 1024 << " KB" << std::endl;
    system("top n 1 b | head -n 4 | tail -n 1");
    std::cout << std::endl;
}


void logMemoryUsage(std::string event)
{
    const std::string filename = "debugMemoryLog.tmp";
    std::ofstream outFile(filename.c_str(), std::ios_base::app);
    outFile << "----------" << std::endl << event << std::endl;
    stxxl::malloc_stats mem_stats;
    outFile << mem_stats;
    outFile.close();
    system(("date >> " + filename).c_str());
    system(("ps -C mst -o rss,%mem --no-headers | head -n 1 >> " + filename).c_str());
    system(("top n 1 b | head -n 4 | tail -n 1 >> " + filename).c_str());
}

#endif // UTILS_H
